@@ -146,6 +146,12 @@ class Message:
         """
         return self._message.partition_key()
 
+    def ordering_key(self):
+        """
+        Get the ordering key for the message.
+        """
+        return self._message.ordering_key()
+
     def publish_timestamp(self):
         """
         Get the timestamp in milliseconds with the message publish time.
@@ -434,6 +440,26 @@ class ConsumerDeadLetterPolicy:
         """
         return self._policy
 
+class CryptoKeyReader:
+    """
+    Default crypto key reader implementation
+    """
+    def __init__(self, public_key_path, private_key_path):
+        """
+        Create crypto key reader.
+
+        Parameters
+        ----------
+
+        public_key_path: str
+            Path to the public key
+        private_key_path: str
+            Path to private key
+        """
+        _check_type(str, public_key_path, 'public_key_path')
+        _check_type(str, private_key_path, 'private_key_path')
+        self.cryptoKeyReader = _pulsar.CryptoKeyReader(public_key_path, private_key_path)
+
 class Client:
     """
     The Pulsar client. A single client instance can be used to create producers
@@ -568,7 +594,7 @@ class Client:
                         schema=schema.BytesSchema(),
                         initial_sequence_id=None,
                         send_timeout_millis=30000,
-                        compression_type=CompressionType.NONE,
+                        compression_type: CompressionType = CompressionType.NONE,
                         max_pending_messages=1000,
                         max_pending_messages_across_partitions=50000,
                         block_if_queue_full=False,
@@ -577,13 +603,13 @@ class Client:
                         batching_max_allowed_size_in_bytes=128*1024,
                         batching_max_publish_delay_ms=10,
                         chunking_enabled=False,
-                        message_routing_mode=PartitionsRoutingMode.RoundRobinDistribution,
+                        message_routing_mode: PartitionsRoutingMode = PartitionsRoutingMode.RoundRobinDistribution,
                         lazy_start_partitioned_producers=False,
                         properties=None,
-                        batching_type=BatchingType.Default,
+                        batching_type: BatchingType = BatchingType.Default,
                         encryption_key=None,
-                        crypto_key_reader=None,
-                        access_mode=ProducerAccessMode.Shared,
+                        crypto_key_reader: CryptoKeyReader = None,
+                        access_mode: ProducerAccessMode = ProducerAccessMode.Shared,
                         ):
         """
         Create a new producer on a given topic.
@@ -746,7 +772,7 @@ class Client:
         return p
 
     def subscribe(self, topic, subscription_name,
-                  consumer_type=ConsumerType.Exclusive,
+                  consumer_type: ConsumerType = ConsumerType.Exclusive,
                   schema=schema.BytesSchema(),
                   message_listener=None,
                   receiver_queue_size=1000,
@@ -758,8 +784,8 @@ class Client:
                   is_read_compacted=False,
                   properties=None,
                   pattern_auto_discovery_period=60,
-                  initial_position=InitialPosition.Latest,
-                  crypto_key_reader=None,
+                  initial_position: InitialPosition = InitialPosition.Latest,
+                  crypto_key_reader: CryptoKeyReader = None,
                   replicate_subscription_state_enabled=False,
                   max_pending_chunked_message=10,
                   auto_ack_oldest_chunked_message_on_queue_full=False,
@@ -767,7 +793,7 @@ class Client:
                   batch_receive_policy=None,
                   key_shared_policy=None,
                   batch_index_ack_enabled=False,
-                  regex_subscription_mode=RegexSubscriptionMode.PersistentOnly,
+                  regex_subscription_mode: RegexSubscriptionMode = RegexSubscriptionMode.PersistentOnly,
                   dead_letter_policy: ConsumerDeadLetterPolicy = None,
                   ):
         """
@@ -960,7 +986,8 @@ class Client:
                       reader_name=None,
                       subscription_role_prefix=None,
                       is_read_compacted=False,
-                      crypto_key_reader=None
+                      crypto_key_reader: CryptoKeyReader = None,
+                      start_message_id_inclusive=False
                       ):
         """
         Create a reader on a particular topic
@@ -1019,8 +1046,10 @@ class Client:
         crypto_key_reader: CryptoKeyReader, optional
             Symmetric encryption class implementation, configuring public key encryption messages for the producer
             and private key decryption messages for the consumer
+        start_message_id_inclusive: bool, default=False
+            Set the reader to include the startMessageId or given position of any reset operation like Reader.seek
         """
-        
+
         # If a pulsar.MessageId object is passed, access the _pulsar.MessageId object
         if isinstance(start_message_id, MessageId):
             start_message_id = start_message_id._msg_id
@@ -1033,6 +1062,7 @@ class Client:
         _check_type_or_none(str, subscription_role_prefix, 'subscription_role_prefix')
         _check_type(bool, is_read_compacted, 'is_read_compacted')
         _check_type_or_none(CryptoKeyReader, crypto_key_reader, 'crypto_key_reader')
+        _check_type(bool, start_message_id_inclusive, 'start_message_id_inclusive')
 
         conf = _pulsar.ReaderConfiguration()
         if reader_listener:
@@ -1046,6 +1076,7 @@ class Client:
         conf.read_compacted(is_read_compacted)
         if crypto_key_reader:
             conf.crypto_key_reader(crypto_key_reader.cryptoKeyReader)
+        conf.start_message_id_inclusive(start_message_id_inclusive)
 
         c = Reader()
         c._reader = self._client.create_reader(topic, start_message_id, conf)
@@ -1142,6 +1173,7 @@ class Producer:
     def send(self, content,
              properties=None,
              partition_key=None,
+             ordering_key=None,
              sequence_id=None,
              replication_clusters=None,
              disable_replication=False,
@@ -1164,6 +1196,8 @@ class Producer:
         partition_key: optional
             Sets the partition key for message routing. A hash of this key is used
             to determine the message's topic partition.
+        ordering_key: optional
+            Sets the ordering key for message routing.
         sequence_id:  optional
             Specify a custom sequence id for the message being published.
         replication_clusters:  optional
@@ -1180,7 +1214,7 @@ class Producer:
         deliver_after: optional
             Specify a delay in timedelta for the delivery of the messages.
         """
-        msg = self._build_msg(content, properties, partition_key, sequence_id,
+        msg = self._build_msg(content, properties, partition_key, ordering_key, sequence_id,
                               replication_clusters, disable_replication, event_timestamp,
                               deliver_at, deliver_after)
         return self._producer.send(msg)
@@ -1188,6 +1222,7 @@ class Producer:
     def send_async(self, content, callback,
                    properties=None,
                    partition_key=None,
+                   ordering_key=None,
                    sequence_id=None,
                    replication_clusters=None,
                    disable_replication=False,
@@ -1239,6 +1274,8 @@ class Producer:
         partition_key: optional
             Sets the partition key for the message routing. A hash of this key is
             used to determine the message's topic partition.
+        ordering_key: optional
+            Sets the ordering key for the message routing.
         sequence_id: optional
             Specify a custom sequence id for the message being published.
         replication_clusters: optional
@@ -1254,7 +1291,7 @@ class Producer:
         deliver_after: optional
             Specify a delay in timedelta for the delivery of the messages.
         """
-        msg = self._build_msg(content, properties, partition_key, sequence_id,
+        msg = self._build_msg(content, properties, partition_key, ordering_key, sequence_id,
                               replication_clusters, disable_replication, event_timestamp,
                               deliver_at, deliver_after)
         self._producer.send_async(msg, callback)
@@ -1274,7 +1311,7 @@ class Producer:
         """
         self._producer.close()
 
-    def _build_msg(self, content, properties, partition_key, sequence_id,
+    def _build_msg(self, content, properties, partition_key, ordering_key, sequence_id,
                    replication_clusters, disable_replication, event_timestamp,
                    deliver_at, deliver_after):
         data = self._schema.encode(content)
@@ -1282,6 +1319,7 @@ class Producer:
         _check_type(bytes, data, 'data')
         _check_type_or_none(dict, properties, 'properties')
         _check_type_or_none(str, partition_key, 'partition_key')
+        _check_type_or_none(str, ordering_key, 'ordering_key')
         _check_type_or_none(int, sequence_id, 'sequence_id')
         _check_type_or_none(list, replication_clusters, 'replication_clusters')
         _check_type(bool, disable_replication, 'disable_replication')
@@ -1296,6 +1334,8 @@ class Producer:
                 mb.property(k, v)
         if partition_key:
             mb.partition_key(partition_key)
+        if ordering_key:
+            mb.ordering_key(ordering_key)
         if sequence_id:
             mb.sequence_id(sequence_id)
         if replication_clusters:
@@ -1693,27 +1733,6 @@ class Reader:
         return self._reader.is_connected()
 
 
-class CryptoKeyReader:
-    """
-    Default crypto key reader implementation
-    """
-    def __init__(self, public_key_path, private_key_path):
-        """
-        Create crypto key reader.
-
-        Parameters
-        ----------
-
-        public_key_path: str
-            Path to the public key
-        private_key_path: str
-            Path to private key
-        """
-        _check_type(str, public_key_path, 'public_key_path')
-        _check_type(str, private_key_path, 'private_key_path')
-        self.cryptoKeyReader = _pulsar.CryptoKeyReader(public_key_path, private_key_path)
-
-
 class ConsoleLogger:
     """
     Logger that writes on standard output
