@@ -19,6 +19,7 @@
 #
 
 
+import random
 import threading
 import logging
 from unittest import TestCase, main
@@ -55,7 +56,9 @@ from schema_test import *
 from urllib.request import urlopen, Request
 
 TM = 10000  # Do not wait forever in tests
-CERTS_DIR = os.path.dirname(os.path.abspath(__file__)) + "/test-conf/"
+TESTS_DIR = os.path.dirname(os.path.abspath(__file__))
+TOKEN_PATH = TESTS_DIR + "/.test-token.txt"
+CERTS_DIR = TESTS_DIR + "/test-conf/"
 
 def doHttpPost(url, data):
     req = Request(url, data.encode())
@@ -251,6 +254,23 @@ class PulsarTest(TestCase):
         consumer.unsubscribe()
         client.close()
 
+    def test_ordering_key(self):
+        client = Client(self.serviceUrl)
+        consumer = client.subscribe(
+            "my-python-topic-ordering-key", "my-sub", consumer_type=ConsumerType.KeyShared
+        )
+        producer = client.create_producer("my-python-topic-ordering-key")
+        producer.send(b"ordered-hello", ordering_key="random-ordering-key")
+
+        # Message should be available immediately with ordering key set
+        msg = consumer.receive(TM)
+        self.assertTrue(msg)
+        self.assertEqual(msg.data(), b"ordered-hello")
+        self.assertEqual(msg.ordering_key(), "random-ordering-key")
+        consumer.unsubscribe()
+        producer.close()
+        client.close()
+
     def test_redelivery_count(self):
         client = Client(self.serviceUrl)
         consumer = client.subscribe(
@@ -266,7 +286,10 @@ class PulsarTest(TestCase):
         for i in range(4):
             msg = consumer.receive(TM)
             print("Received message %s" % msg.data())
-            consumer.negative_acknowledge(msg)
+            if i % 2 == 0:
+                consumer.negative_acknowledge(msg)
+            else:
+                consumer.negative_acknowledge(msg.message_id())
             redelivery_count = msg.redelivery_count()
 
         self.assertTrue(msg)
@@ -495,7 +518,7 @@ class PulsarTest(TestCase):
             authentication=Authentication(authPlugin, authParams),
         )
 
-        with self.assertRaises(pulsar.ConnectError):
+        with self.assertRaises(pulsar.AuthenticationError):
             client.subscribe("my-python-topic-auth-junk-params", "my-sub", consumer_type=ConsumerType.Shared)
 
     def test_message_listener(self):
@@ -668,6 +691,41 @@ class PulsarTest(TestCase):
         self.assertFalse(reader.is_connected())
         client.close()
 
+    def test_reader_seek_for_message_id(self):
+        client = pulsar.Client(self.serviceUrl)
+
+        topic = "test-seek-for-message-id-" + str(int(time.time()))
+
+        producer = client.create_producer(topic)
+
+        readerExclusive = client.create_reader(topic, MessageId.latest)
+        readerInclusive = client.create_reader(topic, MessageId.latest, start_message_id_inclusive=True)
+
+        numMessages = 100
+        seekMessageId = None
+
+        r = random.randint(0, numMessages - 2)
+        for i in range(numMessages):
+            msg_content = b"msg-%d" % i
+            id = producer.send(msg_content)
+
+            if i == r:
+                seekMessageId = id
+
+        readerExclusive.seek(seekMessageId)
+        msg0 = readerExclusive.read_next(timeout_millis=3000)
+
+        readerInclusive.seek(seekMessageId)
+        msg1 = readerInclusive.read_next(timeout_millis=3000)
+
+        self.assertEqual(msg0.data(), b"msg-%d" % (r + 1))
+        self.assertEqual(msg1.data(), b"msg-%d" % r)
+
+        readerExclusive.close()
+        readerInclusive.close()
+        producer.close()
+        client.close()
+
     def test_producer_sequence_after_reconnection(self):
         # Enable deduplication on namespace
         doHttpPost(self.adminUrl + "/admin/v2/namespaces/public/default/deduplication", "true")
@@ -960,19 +1018,16 @@ class PulsarTest(TestCase):
 
         # seek, and after reconnect, expected receive first message.
         consumer.seek(MessageId.earliest)
-        time.sleep(0.5)
         msg = consumer.receive(TM)
         self.assertEqual(msg.data(), b"hello-0")
 
         # seek on messageId
         consumer.seek(ids[50])
-        time.sleep(0.5)
         msg = consumer.receive(TM)
         self.assertEqual(msg.data(), b"hello-51")
 
         # ditto, but seek on timestamp
         consumer.seek(timestamps[42])
-        time.sleep(0.5)
         msg = consumer.receive(TM)
         self.assertEqual(msg.data(), b"hello-42")
 
@@ -983,7 +1038,6 @@ class PulsarTest(TestCase):
 
         # earliest
         reader.seek(MessageId.earliest)
-        time.sleep(0.5)
         msg = reader.read_next(TM)
         self.assertEqual(msg.data(), b"hello-0")
         msg = reader.read_next(TM)
@@ -991,7 +1045,6 @@ class PulsarTest(TestCase):
 
         # seek on messageId
         reader.seek(ids[33])
-        time.sleep(0.5)
         msg = reader.read_next(TM)
         self.assertEqual(msg.data(), b"hello-34")
         msg = reader.read_next(TM)
@@ -999,7 +1052,6 @@ class PulsarTest(TestCase):
 
         # seek on timestamp
         reader.seek(timestamps[79])
-        time.sleep(0.5)
         msg = reader.read_next(TM)
         self.assertEqual(msg.data(), b"hello-79")
         msg = reader.read_next(TM)
@@ -1200,7 +1252,7 @@ class PulsarTest(TestCase):
         client.close()
 
     def test_token_auth(self):
-        with open(".test-token.txt") as tf:
+        with open(TOKEN_PATH) as tf:
             token = tf.read().strip()
 
         # Use adminUrl to test both HTTP request and binary protocol
@@ -1219,7 +1271,7 @@ class PulsarTest(TestCase):
 
     def test_token_auth_supplier(self):
         def read_token():
-            with open(".test-token.txt") as tf:
+            with open(TOKEN_PATH) as tf:
                 return tf.read().strip()
 
         client = Client(self.serviceUrl, authentication=AuthenticationToken(read_token))
